	TITLE	'ATLAS system initialisation module for CP/M 3'

; ATLAS initialisation routines
;
; Copyright (C) 1984, Arcom Control Systems Ltd.
;
; John Fielden 1-Oct-84

; Version 1.5 - 850311
; Miscellaneous initialisation on cold boot and cold and warm boot
; loading of CCP.

	NAME	'BOOT'

FALSE	EQU	0
TRUE	EQU	NOT FALSE

;
BANKED	EQU	FALSE

;
	PUBLIC	?INIT,?LDCCP,?RLCCP

	EXTRN	?PMSG,?CONIN,?BNKSL,?MOV,?XMOV
	EXTRN	@CIVEC,@COVEC,@AIVEC,@AOVEC,@LOVEC
	EXTRN	CRTBD

;

	MACLIB	Z80

;
SCCAC	EQU	5			; SCC channel A control register

;
BDOS	EQU	0005H			; BDOS entry point address

; Usefull ASCII characters
CR	EQU	13
LF	EQU	10
BEL	EQU	7

; Initialisation code can reside in banked memory

	DSEG

; Initialise

?INIT:	MVI	C,SCCAC			; read baud rate generator divisor
	MVI	A,0CH			; lsb
	OUTP	A
	INP	L
	INR	A			; msb
	OUTP	A
	INP	H
	MVI	A,16			; compare with table
	DI
	SSPD	SPSAVE
	LXI	SP,BRTBL
INIT:	DCR	A
	POP	D
	DAD	D
	JRC	INIT
	LSPD	SPSAVE
	EI
	STA	CRTBD			; CRT baud rate
	LXI	H,8000H			; assign console to CRT
	SHLD	@CIVEC
	SHLD	@COVEC
	LXI	H,4000H			; assign printer to LPT
	SHLD	@LOVEC
	SHLD	@AIVEC			; assign AUX to LPT
	SHLD	@AOVEC
	LXI	H,SIGNON		; print greetings
	CALL	?PMSG
	RET

; baud rate comparison table
BRTBL:	DW	    -8,    -5,    -7,    -9,   -13,   -17,   -26,   -69
	DW	  -157,  -312,  -255,  -151,  -371,  -683,-32768
;
SIGNON:	DB	CR,LF,'Arcom ATLAS CP/M 3 BIOS V1.5'
	DB	CR,LF,'Copyright (C) 1984, Arcom Control Systems Ltd.',CR,LF,0
;
SPSAVE:	DW	0			; save stack pointer

; The rest must be permanently resident

	CSEG

; Load the command processor.
; First the default disk is searched, then disk A:, and then the user
; is prompted to replace the system disk in A:.

IF NOT BANKED
?RLCCP:
ENDIF
?LDCCP:	XRA	A
	LXI	D,FCB
	STAX	D			; try default drive first
LDCCP0:	STA	FCBEX			; extent=0
	STA	FCBCR			; start at beginning of file
	MVI	C,15			; open file
	CALL	BDOS
	INR	A			; check that file was found
	JRNZ	LDCCP1
	LXI	H,FCB			; has disk A: been searched?
	CMP	M
	JRNZ	NOCCP
	INR	M			; now try A:
	XCHG
	JR	LDCCP0
LDCCP1:	LXI	D,0100H			; load CCP into TPA
	MVI	C,26
	CALL	BDOS
	LXI	D,128			; allow upto 128 records!
	MVI	C,44
	CALL	BDOS
	LXI	D,FCB			; read the file
	MVI	C,20
	CALL	BDOS
IF BANKED
	XRA	A			; this code and stack are in
	CALL	?BNKSL			; common, so rest can be banked
	CALL	SAVCCP			; save CCP in bank 2
	MVI	A,1			; back to TPA
	JMP	?BNKSL
ELSE
	RET
ENDIF
; file not found
NOCCP:	LXI	H,ERRMSG		; print error message
	CALL	?PMSG
	CALL	?CONIN			; wait for response
	MVI	C,13			; reset disk system
	CALL	BDOS
	JR	?LDCCP			; and try again

IF BANKED
; reload CCP
?RLCCP:	XRA	A			; this code and stack are in
	CALL	?BNKSL			; common, so rest can be banked
	CALL	RESCCP			; restore CCP from bank 2
	MVI	A,1			; back to TPA
	JMP	?BNKSL

	DSEG				; CCP relocate code can be banked

; save CCP. H still contains count of records.
SAVCCP:	MOV	A,H			; remember number of records
	STA	COUNT
	MOV	B,A			; now copy into bank 2
	LXI	D,0100H			; move from bank 1 0100H
	LXI	H,0000H			;      to   bank 2 0000H
SAVCP1:	PUSH	B
	PUSH	D
	PUSH	H
	LXI	B,0201H			; move from bank 1 to 2
	CALL	?XMOV
	POP	H			; destination address
	POP	D			; source address
	LXI	B,128			; move 1 record at a time
	CALL	?MOV
	POP	B
	DJNZ	SAVCP1
	RET
; restore CCP from bank 2.
RESCCP:	LDA	COUNT			; number of records to move
	MOV	B,A
	LXI	D,0000H			; move from bank 2 0000H
	LXI	H,0100H			;      to   bank 1 0100H
RESCP1:	PUSH	B
	PUSH	D
	PUSH	H
	LXI	B,0102H			; move from bank 2 to 1
	CALL	?XMOV
	POP	H			; destination address
	POP	D			; source address
	LXI	B,128			; move 1 record at a time
	CALL	?MOV
	POP	B
	DJNZ	RESCP1
	RET
;
COUNT:	DB	0

	CSEG				; data must be resident
ENDIF

;
ERRMSG:	DB	CR,LF,BEL,'File CCP.COM not found'
	DB	CR,LF,'Insert system disk into A: then hit a key',0
;
FCB:	DB	0,'CCP     ','COM'
FCBEX:	DB	0
	DB	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
FCBCR:	DB	0
	DB	0,0,0

;

	END

