	TITLE	'ATLAS character I/O'

; ATLAS character I/O drivers for CP/M 3 BIOS
;
; Copyright (C) 1984, Arcom Control Systems Ltd.
;
; John Fielden 17-May-84

; All the character I/O routines vector via the EPROM jump tables.
; Hence these routines are independant of the actual hardware and
; assignment of the devices to their device numbers.

	NAME	'CHARIO'

	PUBLIC	?CINIT,?CI,?CO,?CIST,?COST
	PUBLIC	@CTBL,CRTBD

	EXTRN	ESEL			; select EPROM

;

	MACLIB	MODEBAUD
	MACLIB	Z80

; Number of devices
NDEV	EQU	3			; only using SCC and keyboard port

; EPROM jump table addresses
CINI	EQU	0081H			; device initialisation jump table
CIS	EQU	0084H			; input status routines jump table
CIN	EQU	0087H			; input routines jump table
COS	EQU	008AH			; output status routines jump table
COUT	EQU	008DH			; output routines jump table

; All this code must be resident

	CSEG

; initialise specified device
?CINIT:	MOV	A,C			; device number
	CPI	NDEV			; in range?
	RNC
	ADD	A			; * 8
	ADD	A
	ADD	A
	MOV	L,A
	MVI	H,0
	LXI	D,CRTBD			; index into character device table
	DAD	D
	MOV	A,M			; get baud rate
	ANA	A			; if baud rate = 0, don't initialise
	RZ
	MOV	B,C			; device number
	LXI	H,CINI			; address of pointer to jump table
	JR	VECTOR

; character input
?CI:	CALL	?CIST			; character available?
	JRZ	?CI
	LXI	H,CIN			; address of pointer to jump table
	JR	VECTOR

; character input status
?CIST:	LXI	H,CIS			; address of pointer to jump table
	JR	VECTOR

; character output
?CO:	CALL	?COST			; transmitter free?
	JRZ	?CO
	MOV	A,C			; character to output
	LXI	H,COUT			; address of pointer to jump table
	JR	VECTOR

; character output status
?COST:	LXI	H,COS			; address of pointer to jump table
	JR	VECTOR

; vector via EPROM jump tables
; given:    HL = address of address of start of jump table
;           B = device number
;           A = argument to pass
; returned: A = value returned from EPROM routine
VECTOR:	CALL	ESEL			; switch in EPROM
	MOV	E,M			; get address of start of jump table
	INX	H
	MOV	D,M
	MOV	L,B			; index into it
	MVI	H,0
	XCHG
	DAD	D			; device number * 3
	DAD	D
	DAD	D
	PCHL				; call routine, and return via DSEL

;
@CTBL:	DB	'CRT   '		; device 0 is CRT
	DB	MB$IN$OUT+MB$SERIAL+MB$SOFTBAUD
CRTBD:	DB	BAUD$NONE		; default is don't change
	DB	'LPT   '		; device 1 is LPT
	DB	MB$IN$OUT+MB$SERIAL+MB$SOFTBAUD
	DB	BAUD$1200
	DB	'KBD   '		; device 2 is KBD
	DB	MB$INPUT
	DB	BAUD$NONE		; baud rate not meaningful
	DB	0			; end of table

;
	END

