	TITLE	'ATLAS character I/O'

; ATLAS character I/O drivers for CP/M 3 BIOS.
; For a system including Arcom STE MVS board.
;
; Copyright (C) 1984, Arcom Control Systems Ltd.
;
; John Fielden 17-Jul-84

; All the character I/O routines vector via the EPROM jump tables.
; Additional code is included for handling escape sequences sent to
; the video card.
; Hence these routines are independant of the actual hardware and
; assignment of the devices to their device numbers.

	NAME	'CHARIO'

; address of start of MVS code
MVSCOD	EQU	0F400H			; 48x16 set F800H, 80x20 set F400H

	PUBLIC	?CINIT,?CI,?CO,?CIST,?COST
	PUBLIC	@CTBL

	EXTRN	ESEL			; select EPROM
	EXTRN	PORT0			; value of port 0

; MVS routines
MVSRES	EQU	MVSCOD			; reset display
MVSCLR	EQU	MVSCOD+3		; clear screen
MVSPUT	EQU	MVSCOD+6		; put character to screen
MVSCC	EQU	MVSCOD+9		; clear cursor
MVSDC	EQU	MVSCOD+12		; draw cursor
MVSDRW	EQU	MVSCOD+15		; draw screen

; Variables used by EPROM MVS routines
MVSX	EQU	MVSCOD+18		; x-coordinate
MVSY	EQU	MVSCOD+19		; y-coordinate
MVSCS	EQU	MVSCOD+20		; address of cursor on screen
MVSCB	EQU	MVSCOD+22		; address of cursor in buffer
MVSSCR	EQU	MVSCOD+24		; address of start of screen
MVSBUF	EQU	MVSCOD+26		; address of start of buffer
MVSCOL	EQU	MVSCOD+28		; number of columns
MVSWID	EQU	MVSCOD+29		; width of one line in video memory
MVSLEN	EQU	MVSCOD+30		; number of lines on screen
MVSMOD	EQU	MVSCOD+31		; MVS mode (0 = monitor)
MVSCUR	EQU	MVSCOD+32		; MVS cursor character
MVSP0	EQU	MVSCOD+33		; MVS code copy of value of port 0

;

	MACLIB	MODEBAUD
	MACLIB	Z80

; Number of devices
NDEV	EQU	4			; using MVS, SCC and keyboard port

; EPROM jump table addresses
CINI	EQU	0081H			; device initialisation jump table
CIS	EQU	0084H			; input status routines jump table
CIN	EQU	0087H			; input routines jump table
COS	EQU	008AH			; output status routines jump table
COUT	EQU	008DH			; output routines jump table

; All this code must be resident

	CSEG

; initialise specified device
?CINIT:	MOV	A,C			; device number
	CPI	NDEV			; in range?
	RNC
	ANA	A			; device 0?
	JRNZ	CINIT1
	MVI	A,1			; enable inverse video cursor
	STA	MVSCUR
	RET
CINIT1:	ADD	A			; * 8
	ADD	A
	ADD	A
	MOV	L,A
	MVI	H,0
	LXI	D,CRTBD			; index into character device table
	DAD	D
	MOV	A,M			; get baud rate
	ANA	A			; if baud rate = 0, don't initialise
	RZ
	MOV	B,C			; device number
	LXI	H,CINI			; address of pointer to jump table
	JR	VECTOR

; character input
?CI:	CALL	?CIST			; character available?
	JRZ	?CI
	LXI	H,CIN			; address of pointer to jump table
	JR	VECTOR

; character input status
?CIST:	LXI	H,CIS			; address of pointer to jump table
	JR	VECTOR

; character output
?CO:	MOV	A,B
	ANA	A			; CRT?
	JRZ	MVS			; if yes, branch to MVS driver
?CO1:	CALL	?COST			; transmitter free?
	JRZ	?CO1
	MOV	A,C			; character to output
	LXI	H,COUT			; address of pointer to jump table
	JR	VECTOR

; character output status
?COST:	LXI	H,COS			; address of pointer to jump table

; vector via EPROM jump tables
; given:    HL = address of address of start of jump table
;           B = device number
;           A = argument to pass
; returned: A = value returned from EPROM routine
VECTOR:	CALL	ESEL			; switch in EPROM
	PUSH	A			; copy across port 0
	LDA	PORT0
	STA	MVSP0
	POP	A
	MOV	E,M			; get address of start of jump table
	INX	H
	MOV	D,M
	MOV	L,B			; index into it
	MVI	H,0
	XCHG
	DAD	D			; device number * 3
	DAD	D
	DAD	D
	PCHL				; call routine, and return via DSEL

; Driver software for video board.
; This software interprets escape sequences for cursor control.
; The following sequences are implemented:
; ESC=rc where r and c are 2 ASCII characters representing the row
;        and column positions respectively. This places the cursor
;        at the specified position. The coordinate system has its
;        origin (0,0) at the top left-hand corner of the screen and
;        the characters r and c are derived by adding 20H to the
;        coordinate value.
; ESCc  where c represents any control character causes that control
;       character to be printed.
;
; handle one character sent to the MVS
MVS:	LDA	MVSESC			; in escape sequence?
	ANA	A
	JRZ	MVS2
	CPI	0FFH			; yes, first character?
	JRNZ	ESC
	MOV	A,C			; get character
	CPI	'='			; cursor position
	JRZ	MVS1
	LDA	MVSMOD			; print character, first save mode
	PUSH	PSW
	XRA	A			; force monitor mode
	STA	MVSMOD
	MOV	A,C			; character
	CALL	MVS3			; print it
	POP	PSW			; restore mode
	STA	MVSMOD
	XRA	A
MVS1:	STA	MVSESC
	RET
MVS2:	MOV	A,C			; character to output
	CPI	27			; ESC?
	JRNZ	MVS3
	MVI	A,0FFH			; set flag
	STA	MVSESC
	CALL	ESEL			; clear cursor
	LDA	PORT0			; copy across port 0
	STA	MVSP0
	JMP	MVSCC			; call routine and return via DSEL
MVS3:	LXI	H,COUT			; address of pointer to jump table
	JR	VECTOR
; interpret third, or later, character of escape sequence
ESC:	ANI	80H			; cursor position sequence, x or y?
	MOV	A,C
	JRNZ	ESC6
	SUI	20H			; y
	LXI	H,MVSLEN		; check that it is in range
	CMP	M
	JRC	ESC1
	MOV	A,M
	DCR	A
ESC1:	STA	MVSY			; store
	LHLD	MVSSCR			; compute cursor position
	MOV	B,A
	LDA	MVSWID
	MOV	E,A
	MVI	D,0
	MOV	A,B
	ANA	A
	JRZ	ESC3
ESC2:	DAD	D
	DJNZ	ESC2
ESC3:	SHLD	MVSCS
	LHLD	MVSBUF
	MOV	B,A
	LDA	MVSCOL
	MOV	E,A
	MOV	A,B
	ANA	A
	JRZ	ESC5
ESC4:	DAD	D
	DJNZ	ESC4
ESC5:	SHLD	MVSCB
	MVI	A,'='+80H
	STA	MVSESC
	RET
ESC6:	SUI	20H			; x
	LXI	H,MVSCOL		; check that it is in range
	CMP	M
	JRC	ESC7
	MOV	A,M
	DCR	A
ESC7:	STA	MVSX			; store
	MOV	E,A			; compute cursor position
	MVI	D,0
	LHLD	MVSCS
	DAD	D
	SHLD	MVSCS
	LHLD	MVSCB
	DAD	D
	SHLD	MVSCB
;
	XRA	A			; end of escape sequence
	STA	MVSESC
	CALL	ESEL			; draw cursor
	LDA	PORT0			; copy across port 0
	STA	MVSP0
	JMP	MVSDC			; call routine, and return via DSEL

;
@CTBL:	DB	'CRT   '		; device 0 is CRT
	DB	MB$IN$OUT+MB$SERIAL
CRTBD:	DB	BAUD$NONE		; don't change baud rate
	DB	'LPT0  '		; device 1 is LPT
	DB	MB$IN$OUT+MB$SERIAL+MB$SOFTBAUD
	DB	BAUD$1200
	DB	'KBD   '		; device 2 is KBD
	DB	MB$INPUT
	DB	BAUD$NONE		; baud rate not meaningful
	DB	'LPT1  '		; device 3 is spare serial output
	DB	MB$OUTPUT
	DB	BAUD$NONE		; don't change baud rate
	DB	0			; end of table

;
MVSESC:	DB	0			; escape sequence state

;
	END

