	TITLE	'ATLAS floppy disk parameter headers'

; Floppy disk parameter headers, parameter blocks and
; other miscellaneous drive configuration data.

; Copyright (C) 1984, Arcom Control Systems Ltd.

; Configuration for different drive types can be done
; by changing this file without having to make any changes to
; the source code of the floppy disk routines.

; Please note that the logical starting sector number of a disk
; should be 0 (this is translate to a physical sector number by the
; BIOS). Hence the standard 8-inch single-sided single-density translate
; table is produced by the following invocation of the SKEW macro:
;
;	SKEW	26,6,0
;

	MACLIB	CPM3

; Disk drive dispatching tables
	PUBLIC	FDSK0,FDSK1,FDSK2

; Other data needed by FLOPPY
	PUBLIC	FNTRKS,FLSECS

; Routines in FLOPPY referenced by FDPH
	EXTRN	FINIT,FLOGIN,FREAD,FWRITE

; banked data

	DSEG

; Extended disk parameter headers

; The type byte contains information relating to the characteristics
; of the drive and disk. The bits are assigned as follows:
;
; bit 7: 1 = mini-floppy drive, 1 = 8"-floppy drive
; bits 6,5: drive select bits (as latch at I/O address 0)
;             10 => drive 0
;	      01 => drive 1
;	      00 => drive 2 (requires the Arcom disk expansion card)
;	      11 => drive 3 (requires the Arcom disk expansion card)
; bit 4: 1 = current disk is single sided, 0 = current disk is double sided
; bit 3: 0 = current disk is double density, 1 = current disk is single density
; bit 2: 0 = drive is single sided, 1 = drive is double sided
; bits 0,1: = step speed bits as in WD279x type I command:
;             00 =>  3 ms step rate
;             01 =>  6 ms step rate
;             10 => 10 ms step rate
;
; The value of TYPE = 0FFH is reserved and should not be used
; for any floppy disk drive.

; NB in order that GENCPM can work, the DPH and DPB for 8-inch drives
; must be set up for double-density disks.

; floppy drive 0
	DW	FWRITE
	DW	FREAD
	DW	FLOGIN
	DW	FINIT
	DB	0,44H			; 5.25" double-sided drive 0, 3 ms/step
FDSK0:	DPH	0,DPB5DS

; floppy drive 1
	DW	FWRITE
	DW	FREAD
	DW	FLOGIN
	DW	FINIT
	DB	1,24H			; 5.25" double-sided drive 1, 3 ms/step
FDSK1:	DPH	0,DPB5DS

; floppy drive 2
	DW	FWRITE
	DW	FREAD
	DW	FLOGIN
	DW	FINIT
	DB	2,98H			; 8" single-sided drive 1, 3 ms/step
FDSK2:	DPH	TSSSD8,DPB8SD

; translation tables
TSSSD8:	SKEW	26,6,0

; miscellaneous data

; number of tracks on each drive
FNTRKS:	DB	160			; drive 0, 5.25" DS DD
	DB	160			; drive 1, 5.25" DS DD
	DB	77			; drive 2, 8" SS SD

; sector size on each drive
FLSECS:	DW	512			; drive 0, 5.25" DS DD
	DW	512			; drive 1, 5.25" DS DD
	DW	128			; drive 2, 8" SS SD

; DPB's must be resident
	CSEG


; 5.25-inch floppy double-sided double-density 512-byte sectors
DPB5DS:	DPB	512,9,160,4096,128,1

; 8-inch floppy single-sided single-density 128-byte sectors
DPB8SD:	DPB	128,26,77,1024,64,2

;
	END

