	TITLE	'SCPUA/SCPUB real-time clock routines'

; Time and date routines for CP/M 3 BIOS
;
; These routines use the Arcom SCRAM STEbus clock and ram card.
;
; John Fielden 02-Nov-84
; Corrected 14-Feb-85 J.F.
;
; Copyright (C) 1984, Arcom Control Systems Ltd.

	NAME	'TIME'

; Clock I/O addresses

CLOCK	EQU	0F80H
CSECS	EQU	CLOCK
CMINS	EQU	CLOCK+2
CHRS	EQU	CLOCK+4
CDAYS	EQU	CLOCK+7
CMONS	EQU	CLOCK+8
CYEAR	EQU	CLOCK+9
CREGA	EQU	CLOCK+10
CREGB	EQU	CLOCK+11
CREGC	EQU	CLOCK+12
CREGD	EQU	CLOCK+13

;

	PUBLIC	?TIME
;

	EXTRN	@SEC,@MIN,@HOUR,@DATE

;

	MACLIB	Z80

	CSEG

;
?TIME:	PUSH	H
	PUSH	D
	MOV	A,C
	ANA	A
	JRNZ	TSET
; read the clock
TREAD:	LXI	B,CREGA			; check for UIP
	INP	A
	ANI	80H
	JRNZ	TREAD
	MVI	C,LOW CSECS		; seconds
	INP	A
	STA	@SEC
	INR	C			; minutes
	INR	C
	INP	A
	STA	@MIN
	INR	C			; hours
	INR	C
	INP	A
	STA	@HOUR
	INR	C			; days
	INR	C
	INR	C
	INP	A
	LXI	H,0
TREAD1:	INR	L
	SBI	1
	DAA
	JRNZ	TREAD1
	INR	C			; months
	INP	A
	MVI	B,0
	LXI	D,MONTHS
TREAD2:	SBI	1
	DAA
	JRZ	TREAD3
	XCHG
	MOV	C,M
	INX	H
	XCHG
	DAD	B
	JR	TREAD2
TREAD3:	LXI	B,CYEAR			; years
	INP	A
TREAD4:	CPI	78H
	JRZ	TREAD7
	MOV	D,A
	ANI	10H			; leap year?
	RRC
	RRC
	RRC
	ADD	D
	ANI	3
	JRNZ	TREAD6
	MOV	A,H			; check if in Jan or Feb of leap year
	ANA	A
	JNZ	TREAD5
	MOV	A,L
	CPI	61
	JRC	TREAD6
TREAD5:	INX	H
TREAD6:	MOV	A,D
	LXI	D,365
	DAD	D
	DCR	A
	DAA
	JR	TREAD4
TREAD7:	SHLD	@DATE
	JR	TEXIT
; set the clock
TSET:	LXI	B,CREGB			; stop clock updates
	MVI	A,82H
	OUTP	A
	DCR	C			; set divisors correctly
	MVI	A,20H
	OUTP	A
	LHLD	@DATE			; date
	DCX	H			; make it zero relative
	MVI	B,77H			; reference year - 1
	LXI	D,365
TSET1:	MOV	A,B			; increment year
	INR	A
	DAA
	MOV	B,A
	ANI	10H			; leap year?
	RRC
	RRC
	RRC
	ADD	B
	ANI	3
	CPI	1			; set carry if leap year
	DSBC	D
	JRNC	TSET1
	DAD	D			; correct for extra subtraction
	ANA	A			; leap year?
	PUSH	A			; remember if leap year
	JRNZ	TSET2
	INX	H
TSET2:	MOV	A,B			; year
	LXI	B,CYEAR
	OUTP	A
	XRA	A			; month
	MOV	B,A
	LXI	D,MONTHS
TSET3:	INR	A
	DAA
	XCHG
	MOV	C,M
	INX	H
	XCHG
	CPI	2			; Feb?
	JRNZ	TSET4
	POP	A
	PUSH	A
	MVI	A,2
	JRNZ	TSET4			; leap year?
	INX	B
TSET4:	ANA	A
	DSBC	B
	JRNC	TSET3
	DAD	B			; correct for extra subraction
	LXI	B,CMONS
	OUTP	A
	POP	A			; discard leap year flag
	DCR	C			; day
	XRA	A
	INR	L
TSET5:	INR	A
	DAA
	DCR	L
	JRNZ	TSET5
	OUTP	A
	DCR	C			; hours
	DCR	C
	DCR	C
	LDA	@HOUR
	OUTP	A
	DCR	C			; minutes
	DCR	C
	LDA	@MIN
	OUTP	A
	DCR	C			; seconds
	DCR	C
	LDA	@SEC
	OUTP	A
	MVI	C,LOW CREGB
	MVI	A,02			; start clock
	OUTP	A
TEXIT:	POP	D
	POP	H
	RET
;
MONTHS:	DB	31,28,31,30,31,30,31,31,30,31,30,31
;
	END

